@@ -55,37 +55,65 @@ void php_imagick_cleanup_progress_callback(php_imagick_callback* progress_callba
 			efree(progress_callback->previous_callback);
 		}
 
+#ifdef ZEND_ENGINE_3
 		zval_ptr_dtor(&progress_callback->user_callback);
+#else
+		zval_ptr_dtor(&progress_callback->user_callback);
+#endif
 	}
 }
 
 MagickBooleanType php_imagick_progress_monitor_callable(const char *text, const MagickOffsetType offset, const MagickSizeType span, void *userData)
 {
+	int error;
+	zend_fcall_info fci;
+	zend_fcall_info_cache fci_cache;
+
+#ifdef ZEND_ENGINE_3
+	zval zargs[2];
+	zval retval;
+#else
+	zval **zargs[2];
+	zval *retval_ptr;
+#endif
+
 	//We can get the data both via the passed param and via
 	//IMAGICK_G(progress_callback) - this should be quicker
 	php_imagick_callback *callback = (php_imagick_callback*)userData;
-	int error;
-	zval **zargs[2];
 
-	zend_fcall_info_cache fci_cache;
-
-	zend_fcall_info fci;
-	zval *retval_ptr = NULL;
+#if defined(ZEND_ENGINE_3) && defined(ZTS)
+	if( NULL == tsrm_get_ls_cache() ) {
+		return MagickTrue;
+	}
+#endif
 
+#ifndef ZEND_ENGINE_3
 	TSRMLS_FETCH_FROM_CTX(callback->thread_ctx);
+#endif
 	fci_cache = empty_fcall_info_cache;
-	retval_ptr = NULL;
 
 	fci.size = sizeof(fci);
 	fci.function_table = EG(function_table);
+#ifdef ZEND_ENGINE_3
+	fci.object = NULL;
+	//fci.function_name = *callback->user_callback;
+	ZVAL_COPY_VALUE(&fci.function_name, &callback->user_callback);
+	fci.retval = &retval;
+#else
+	retval_ptr = NULL;
 	fci.object_ptr = NULL;
 	fci.function_name = callback->user_callback;
 	fci.retval_ptr_ptr = &retval_ptr;
+#endif
 	fci.param_count = 2;
 	fci.params = zargs;
 	fci.no_separation = 0;
 	fci.symbol_table = NULL;
 
+#ifdef ZEND_ENGINE_3
+	ZVAL_LONG(&zargs[0], offset);
+    ZVAL_LONG(&zargs[1], span);
+#else
 	zargs[0] = emalloc(sizeof(zval *));
 	MAKE_STD_ZVAL(*zargs[0]);
 	ZVAL_LONG(*zargs[0], offset);
@@ -93,6 +121,7 @@ MagickBooleanType php_imagick_progress_monitor_callable(const char *text, const
 	zargs[1] = emalloc(sizeof(zval *));
 	MAKE_STD_ZVAL(*zargs[1]);
 	ZVAL_LONG(*zargs[1], span);
+#endif
 
 	error = zend_call_function(&fci, &fci_cache TSRMLS_CC);
 
@@ -101,7 +130,12 @@ MagickBooleanType php_imagick_progress_monitor_callable(const char *text, const
 		//Abort processing as user callback is no longer callable
 		return MagickFalse;
 	}
-
+#ifdef ZEND_ENGINE_3
+	if (Z_TYPE(retval) == IS_FALSE) {
+		//User returned false - tell Imagick to abort processing.
+		return MagickFalse;
+	}
+#else
 	zval_ptr_dtor(zargs[0]);
 	zval_ptr_dtor(zargs[1]);
 
@@ -113,13 +147,26 @@ MagickBooleanType php_imagick_progress_monitor_callable(const char *text, const
 			}
 		}
 	}
+#endif
 
 	return MagickTrue;
 }
 
-zend_bool php_imagick_thumbnail_dimensions(MagickWand *magick_wand, zend_bool bestfit, long desired_width, long desired_height, long *new_width, long *new_height)
+/* This is not universally safe to use, but is safe enough for values that will 
+   be encountered for image dimensions.
+*/
+static inline double im_round_helper(double value) {
+	if (value >= 0.0) {
+		return floor(value + 0.5);
+	} else {
+		return ceil(value - 0.5);
+	}
+}
+
+
+zend_bool php_imagick_thumbnail_dimensions(MagickWand *magick_wand, zend_bool bestfit, im_long desired_width, im_long desired_height, im_long *new_width, im_long *new_height, zend_bool legacy)
 {
-	long orig_width, orig_height;
+	im_long orig_width, orig_height;
 
 	orig_width  = MagickGetImageWidth(magick_wand);
 	orig_height = MagickGetImageHeight(magick_wand);
@@ -146,10 +193,20 @@ zend_bool php_imagick_thumbnail_dimensions(MagickWand *magick_wand, zend_bool be
 			*new_height = desired_height;
 		} else if (ratio_x < ratio_y) {
 			*new_width  = desired_width;
-			*new_height = ratio_x * ((double) orig_height);
+			if (legacy) {
+				*new_height = ratio_x * ((double) orig_height);
+			}
+			else {
+				*new_height = im_round_helper(ratio_x * ((double) orig_height));
+			}
 		} else {
 			*new_height = desired_height;
-			*new_width  = ratio_y * ((double) orig_width);
+			if (legacy) {
+				*new_width  = ratio_y * ((double) orig_width);
+			}
+			else {
+				*new_width  = im_round_helper(ratio_y * ((double) orig_width));
+			}
 		}
 		*new_width  = (*new_width < 1)  ? 1 : *new_width;
 		*new_height = (*new_height < 1) ? 1 : *new_height;
@@ -164,11 +221,21 @@ zend_bool php_imagick_thumbnail_dimensions(MagickWand *magick_wand, zend_bool be
 		if (desired_width <= 0 || desired_height <= 0) {
 			if (desired_width <= 0) {
 				ratio       = (double) orig_height / (double) desired_height;
-				*new_width  = ((double) orig_width) / ratio;
+				if (legacy) {
+					*new_width  = ((double) orig_width) / ratio;
+				}
+				else {
+					*new_width  = im_round_helper(((double) orig_width) / ratio);
+				}
 				*new_height = desired_height;
 			} else {
 				ratio       = (double) orig_width / (double) desired_width;
-				*new_height = ((double) orig_height) / ratio;
+				if (legacy) {
+					*new_height = ((double) orig_height) / ratio;
+				}
+				else {
+					*new_height = im_round_helper(((double) orig_height) / ratio);
+				}
 				*new_width  = desired_width;
 			}
 		} else {
@@ -205,12 +272,17 @@ zend_bool php_imagick_validate_map(const char *map TSRMLS_DC)
 	return 1;
 }
 
-double *php_imagick_zval_to_double_array(zval *param_array, long *num_elements TSRMLS_DC)
+double *php_imagick_zval_to_double_array(zval *param_array, im_long *num_elements TSRMLS_DC)
 {
-	zval **ppzval;
 	double *double_array;
 	long i = 0;
 
+#ifdef ZEND_ENGINE_3
+	zval *pzvalue;
+#else
+	zval **ppzval;
+#endif
+
 	*num_elements = zend_hash_num_elements(Z_ARRVAL_P(param_array));
 
 	if (*num_elements == 0) {
@@ -219,6 +291,13 @@ double *php_imagick_zval_to_double_array(zval *param_array, long *num_elements T
 
 	double_array = ecalloc(*num_elements, sizeof(double));
 
+#ifdef ZEND_ENGINE_3
+	ZEND_HASH_FOREACH_VAL(Z_ARRVAL_P(param_array), pzvalue) {
+		ZVAL_DEREF(pzvalue);
+		double_array[i] = zval_get_double(pzvalue);
+		i++;
+	} ZEND_HASH_FOREACH_END();
+#else
 	for (zend_hash_internal_pointer_reset(Z_ARRVAL_P(param_array));
 			zend_hash_get_current_data(Z_ARRVAL_P(param_array), (void **) &ppzval) == SUCCESS;
 			zend_hash_move_forward(Z_ARRVAL_P(param_array)), i++)
@@ -240,15 +319,21 @@ double *php_imagick_zval_to_double_array(zval *param_array, long *num_elements T
 		}
 		double_array[i] = value;
 	}
+#endif
 	return double_array;
 }
 
 long *php_imagick_zval_to_long_array(zval *param_array, long *num_elements TSRMLS_DC)
 {
-	zval **ppzval;
 	long *long_array;
 	long i = 0;
 
+#ifdef ZEND_ENGINE_3
+	zval *pzvalue;
+#else
+	zval **ppzval;
+#endif
+
 	*num_elements = zend_hash_num_elements(Z_ARRVAL_P(param_array));
 
 	if (*num_elements == 0) {
@@ -257,12 +342,19 @@ long *php_imagick_zval_to_long_array(zval *param_array, long *num_elements TSRML
 
 	long_array = ecalloc(*num_elements, sizeof(long));
 
+#ifdef ZEND_ENGINE_3
+	ZEND_HASH_FOREACH_VAL(Z_ARRVAL_P(param_array), pzvalue) {
+		ZVAL_DEREF(pzvalue);
+		long_array[i] = zval_get_long(pzvalue);
+		i++;
+	} ZEND_HASH_FOREACH_END();
+#else
 	for (zend_hash_internal_pointer_reset(Z_ARRVAL_P(param_array));
 			zend_hash_get_current_data(Z_ARRVAL_P(param_array), (void **) &ppzval) == SUCCESS;
 			zend_hash_move_forward(Z_ARRVAL_P(param_array)), i++)
 	{
 		zval tmp_zval, *tmp_pzval;
-		long value = 0;
+ 		long value = 0;
 
 		if (Z_TYPE_PP(ppzval) == IS_DOUBLE) {
 			value = Z_LVAL_PP(ppzval);
@@ -271,22 +363,29 @@ long *php_imagick_zval_to_long_array(zval *param_array, long *num_elements TSRML
 			tmp_zval = **ppzval;
 			zval_copy_ctor(&tmp_zval);
 			tmp_pzval = &tmp_zval;
-			convert_to_double(tmp_pzval);
+			convert_to_long(tmp_pzval);
 
 			value = Z_LVAL_P(tmp_pzval);
 			zval_dtor (tmp_pzval);
 		}
 		long_array[i] = value;
 	}
+#endif
+
 	return long_array;
 }
 
 unsigned char *php_imagick_zval_to_char_array(zval *param_array, long *num_elements TSRMLS_DC)
 {
-	zval **ppzval;
 	unsigned char *char_array;
 	long i = 0;
 
+#ifdef ZEND_ENGINE_3
+	zval *pzvalue;
+#else
+	zval **ppzval;
+#endif
+
 	*num_elements = zend_hash_num_elements(Z_ARRVAL_P(param_array));
 
 	if (*num_elements == 0) {
@@ -295,13 +394,19 @@ unsigned char *php_imagick_zval_to_char_array(zval *param_array, long *num_eleme
 
 	char_array = ecalloc(*num_elements, sizeof(unsigned char));
 
+#ifdef ZEND_ENGINE_3
+	ZEND_HASH_FOREACH_VAL(Z_ARRVAL_P(param_array), pzvalue) {
+		ZVAL_DEREF(pzvalue);
+		char_array[i] = zval_get_long(pzvalue);
+		i++;
+	} ZEND_HASH_FOREACH_END();
+#else
 	for (zend_hash_internal_pointer_reset(Z_ARRVAL_P(param_array));
 			zend_hash_get_current_data(Z_ARRVAL_P(param_array), (void **) &ppzval) == SUCCESS;
 			zend_hash_move_forward(Z_ARRVAL_P(param_array)), i++)
 	{
 		zval tmp_zval, *tmp_pzval;
 		long value = 0;
-
 		if (Z_TYPE_PP(ppzval) == IS_DOUBLE) {
 			value = Z_LVAL_PP(ppzval);
 		}
@@ -309,13 +414,15 @@ unsigned char *php_imagick_zval_to_char_array(zval *param_array, long *num_eleme
 			tmp_zval = **ppzval;
 			zval_copy_ctor(&tmp_zval);
 			tmp_pzval = &tmp_zval;
-			convert_to_double(tmp_pzval);
+			convert_to_long(tmp_pzval);
 
 			value = Z_LVAL_P(tmp_pzval);
 			zval_dtor (tmp_pzval);
 		}
 		char_array[i] = value;
 	}
+#endif
+
 	return char_array;
 }
 
@@ -421,9 +528,16 @@ PointInfo *php_imagick_zval_to_pointinfo_array(zval *coordinate_array, int *num_
 {
 	PointInfo *coordinates;
 	long elements, sub_elements, i;
+	HashTable *sub_array;
+
+#ifdef ZEND_ENGINE_3
+	zval *pzvalue;
+#else
 	HashTable *coords;
 	zval **ppzval;
-	HashTable *sub_array;
+#endif
+
+	i = 0;
 
 	elements = zend_hash_num_elements(Z_ARRVAL_P(coordinate_array));
 
@@ -436,6 +550,52 @@ PointInfo *php_imagick_zval_to_pointinfo_array(zval *coordinate_array, int *num_
 	*num_elements = elements;
 	coordinates = emalloc(sizeof(PointInfo) * elements);
 
+#ifdef ZEND_ENGINE_3
+	ZEND_HASH_FOREACH_VAL(Z_ARRVAL_P(coordinate_array), pzvalue) {
+		zval *pz_x, *pz_y;
+		ZVAL_DEREF(pzvalue);
+
+		/* If its something than array lets error here */
+		if(Z_TYPE_P(pzvalue) != IS_ARRAY) {
+			efree(coordinates);
+			*num_elements = 0;
+			return NULL;
+		}
+
+		/* Subarray should have two elements. X and Y */
+		sub_elements = zend_hash_num_elements(Z_ARRVAL_P(pzvalue));
+
+		/* Exactly two elements */
+		if (sub_elements != 2) {
+			efree(coordinates);
+			*num_elements = 0;
+			return NULL;
+		}
+
+		/* Subarray values */
+		sub_array = Z_ARRVAL_P(pzvalue);
+
+		/* Get X */
+		if ((pz_x = zend_hash_str_find(sub_array, "x", sizeof("x")-1)) == NULL) {
+			efree(coordinates);
+			*num_elements = 0;
+			return NULL;
+		}
+
+		/* Get Y */
+		if ((pz_y = zend_hash_str_find(sub_array, "y", sizeof("y")-1)) == NULL) {
+			efree(coordinates);
+			*num_elements = 0;
+			return NULL;
+		}
+
+		/* Assign X and Y */
+		coordinates[i].x = zval_get_double(pz_x);
+		coordinates[i].y = zval_get_double(pz_y);
+		i++;
+	} ZEND_HASH_FOREACH_END();
+	
+#else
 	coords = Z_ARRVAL_P(coordinate_array);
 	zend_hash_internal_pointer_reset_ex(coords, (HashPosition *) 0);
 
@@ -494,6 +654,7 @@ PointInfo *php_imagick_zval_to_pointinfo_array(zval *coordinate_array, int *num_
 		coordinates[i].x = Z_DVAL(tmp_zx);
 		coordinates[i].y = Z_DVAL(tmp_zy);
 	}
+#endif
 
 	return coordinates;
 }
@@ -601,6 +762,9 @@ PixelWand *php_imagick_zval_to_pixelwand (zval *param, php_imagick_class_type_t
 	PixelWand *pixel_wand = NULL;
 	*allocated = 0;
 
+#ifdef ZEND_ENGINE_3
+	ZVAL_DEREF(param);
+#endif 
 	if (Z_TYPE_P (param) == IS_LONG || Z_TYPE_P (param) == IS_DOUBLE) {
 		zval var;
 		var = *param;
@@ -629,7 +793,7 @@ PixelWand *php_imagick_zval_to_pixelwand (zval *param, php_imagick_class_type_t
 
 		case IS_OBJECT:
 			if (instanceof_function_ex(Z_OBJCE_P(param), php_imagickpixel_sc_entry, 0 TSRMLS_CC)) {
-				php_imagickpixel_object *intern = (php_imagickpixel_object *)zend_object_store_get_object(param TSRMLS_CC);
+				php_imagickpixel_object *intern = Z_IMAGICKPIXEL_P(param);
 				pixel_wand = intern->pixel_wand;
 			} else
 				php_imagick_throw_exception(caller, "The parameter must be an instance of ImagickPixel or a string" TSRMLS_CC);
@@ -646,6 +810,9 @@ PixelWand *php_imagick_zval_to_opacity (zval *param, php_imagick_class_type_t ca
 	PixelWand *pixel_wand = NULL;
 	*allocated = 0;
 
+#ifdef ZEND_ENGINE_3
+	ZVAL_DEREF(param);
+#endif
 	if (Z_TYPE_P (param) == IS_STRING) {
 		zval var;
 		var = *param;
@@ -663,14 +830,20 @@ PixelWand *php_imagick_zval_to_opacity (zval *param, php_imagick_class_type_t ca
 			if (!pixel_wand) {
 				zend_error(E_ERROR, "Failed to allocate PixelWand structure");
 			}
+#if MagickLibVersion >= 0x700
+			//TOOD - this should be one minus? Or should we just make a BC break
+			// and make users do it in user land?
+			PixelSetAlpha(pixel_wand, Z_DVAL_P(param));
+#else
 			PixelSetOpacity(pixel_wand, Z_DVAL_P(param));
+#endif
 			*allocated = 1;
 		}
 		break;
 
 		case IS_OBJECT:
 			if (instanceof_function_ex(Z_OBJCE_P(param), php_imagickpixel_sc_entry, 0 TSRMLS_CC)) {
-				php_imagickpixel_object *intern = (php_imagickpixel_object *)zend_object_store_get_object(param TSRMLS_CC);
+				php_imagickpixel_object *intern = Z_IMAGICKPIXEL_P(param);
 				pixel_wand = intern->pixel_wand;
 			} else
 				php_imagick_throw_exception(caller, "The parameter must be an instance of ImagickPixel or a string"  TSRMLS_CC);
@@ -789,7 +962,9 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("COLOR_RED", PHP_IMAGICK_COLOR_RED);
 	IMAGICK_REGISTER_CONST_LONG("COLOR_YELLOW", PHP_IMAGICK_COLOR_YELLOW);
 	IMAGICK_REGISTER_CONST_LONG("COLOR_MAGENTA", PHP_IMAGICK_COLOR_MAGENTA);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("COLOR_OPACITY", PHP_IMAGICK_COLOR_OPACITY);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("COLOR_ALPHA", PHP_IMAGICK_COLOR_ALPHA);
 	IMAGICK_REGISTER_CONST_LONG("COLOR_FUZZ", PHP_IMAGICK_COLOR_FUZZ);
 
@@ -812,7 +987,9 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_DEFAULT", OverCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_UNDEFINED", UndefinedCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_NO", NoCompositeOp);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_ADD", AddCompositeOp);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_ATOP", AtopCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_BLEND", BlendCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_BUMPMAP", BumpmapCompositeOp);
@@ -826,7 +1003,13 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_COPYCYAN", CopyCyanCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_COPYGREEN", CopyGreenCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_COPYMAGENTA", CopyMagentaCompositeOp);
+#if MagickLibVersion >= 0x700
+	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_COPYALPHA", CopyAlphaCompositeOp);
+	//TOOD - is this semantically correct?
+	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_COPYOPACITY", CopyAlphaCompositeOp);
+#else
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_COPYOPACITY", CopyOpacityCompositeOp);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_COPYRED", CopyRedCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_COPYYELLOW", CopyYellowCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_DARKEN", DarkenCompositeOp);
@@ -844,7 +1027,9 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_IN", InCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_LIGHTEN", LightenCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_LUMINIZE", LuminizeCompositeOp);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_MINUS", MinusCompositeOp);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_MODULATE", ModulateCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_MULTIPLY", MultiplyCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_OUT", OutCompositeOp);
@@ -860,7 +1045,9 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_SRCIN", SrcInCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_SRCOUT", SrcOutCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_SRCOVER", SrcOverCompositeOp);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_SUBTRACT", SubtractCompositeOp);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_THRESHOLD", ThresholdCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_XOR", XorCompositeOp);
 #if MagickLibVersion >= 0x634
@@ -868,8 +1055,10 @@ void php_imagick_initialize_constants(TSRMLS_D)
 #endif
 #if MagickLibVersion >= 0x636
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_LINEARLIGHT", LinearLightCompositeOp);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_DIVIDE", DivideCompositeOp);
 #endif
+#endif
 #if MagickLibVersion >= 0x654
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_DISTORT", DistortCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_BLUR", BlurCompositeOp);
@@ -894,6 +1083,10 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_DARKENINTENSITY", DarkenIntensityCompositeOp);
 	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_LIGHTENINTENSITY", LightenIntensityCompositeOp);
 #endif
+#if MagickLibVersion >= 0x690
+	IMAGICK_REGISTER_CONST_LONG("COMPOSITE_HARDMIX", HardMixCompositeOp);
+#endif
+
 	IMAGICK_REGISTER_CONST_LONG("MONTAGEMODE_FRAME", FrameMode);
 	IMAGICK_REGISTER_CONST_LONG("MONTAGEMODE_UNFRAME", UnframeMode);
 	IMAGICK_REGISTER_CONST_LONG("MONTAGEMODE_CONCATENATE", ConcatenateMode);
@@ -949,15 +1142,25 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_UNDEFINED", UndefinedType);
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_BILEVEL", BilevelType);
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_GRAYSCALE", GrayscaleType);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_GRAYSCALEMATTE", GrayscaleMatteType);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTE",  PaletteType);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTEMATTE", PaletteMatteType);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_TRUECOLOR", TrueColorType);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_TRUECOLORMATTE", TrueColorMatteType);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_COLORSEPARATION", ColorSeparationType);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_COLORSEPARATIONMATTE", ColorSeparationMatteType);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_OPTIMIZE", OptimizeType);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("IMGTYPE_PALETTEBILEVELMATTE", PaletteBilevelMatteType);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("RESOLUTION_UNDEFINED", UndefinedResolution);
 	IMAGICK_REGISTER_CONST_LONG("RESOLUTION_PIXELSPERINCH", PixelsPerInchResolution);
 	IMAGICK_REGISTER_CONST_LONG("RESOLUTION_PIXELSPERCENTIMETER", PixelsPerCentimeterResolution);
@@ -1005,8 +1208,14 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("GRAVITY_SOUTHWEST", SouthWestGravity);
 	IMAGICK_REGISTER_CONST_LONG("GRAVITY_SOUTH", SouthGravity);
 	IMAGICK_REGISTER_CONST_LONG("GRAVITY_SOUTHEAST", SouthEastGravity);
+	IMAGICK_REGISTER_CONST_LONG("GRAVITY_FORGET", ForgetGravity);
+#if MagickLibVersion < 0x700
+	IMAGICK_REGISTER_CONST_LONG("GRAVITY_STATIC", StaticGravity);
+#endif
+
 	IMAGICK_REGISTER_CONST_LONG("STRETCH_NORMAL", NormalStretch);
 	IMAGICK_REGISTER_CONST_LONG("STRETCH_ULTRACONDENSED", UltraCondensedStretch);
+	IMAGICK_REGISTER_CONST_LONG("STRETCH_EXTRACONDENSED", ExtraCondensedStretch);
 	IMAGICK_REGISTER_CONST_LONG("STRETCH_CONDENSED", CondensedStretch);
 	IMAGICK_REGISTER_CONST_LONG("STRETCH_SEMICONDENSED", SemiCondensedStretch);
 	IMAGICK_REGISTER_CONST_LONG("STRETCH_SEMIEXPANDED", SemiExpandedStretch);
@@ -1042,7 +1251,9 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_YELLOW", YellowChannel);
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_ALPHA", AlphaChannel);
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_OPACITY", OpacityChannel);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_MATTE", MatteChannel); /* deprecated, needs to throw E_STRICT if used */
+#endif
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_BLACK", BlackChannel);
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_INDEX", IndexChannel);
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_ALL", AllChannels);
@@ -1053,27 +1264,55 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_RGBS", RGBChannels);
 #endif
 #if MagickLibVersion >= 0x655
-	IMAGICK_REGISTER_CONST_LONG("CHANNEL_GRAY", GrayChannels);
+	IMAGICK_REGISTER_CONST_LONG("CHANNEL_GRAY_CHANNELS", GrayChannels);
 #endif
 #if MagickLibVersion >= 0x656
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_SYNC", SyncChannels);
 #endif
+
+#if MagickLibVersion >= 0x700
+	IMAGICK_REGISTER_CONST_LONG("CHANNEL_READ_MASK", ReadMaskChannel);          /* Pixel is Not Readable? */
+	IMAGICK_REGISTER_CONST_LONG("CHANNEL_WRITE_MASK", WriteMaskChannel);         /* Pixel is Write Protected? */
+	IMAGICK_REGISTER_CONST_LONG("CHANNEL_META", MetaChannel);              /* ???? */
+#endif
+
 #if MagickLibVersion >= 0x670
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_COMPOSITES", CompositeChannels);
 #endif
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("METRIC_UNDEFINED", UndefinedMetric);
+#endif
+	IMAGICK_REGISTER_CONST_LONG("METRIC_ABSOLUTEERRORMETRIC", AbsoluteErrorMetric);
 	IMAGICK_REGISTER_CONST_LONG("METRIC_MEANABSOLUTEERROR", MeanAbsoluteErrorMetric);
+#if MagickLibVersion < 0x700
+	IMAGICK_REGISTER_CONST_LONG("METRIC_MEANERRORPERPIXELMETRIC", MeanErrorPerPixelMetric);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("METRIC_MEANSQUAREERROR", MeanSquaredErrorMetric);
 	IMAGICK_REGISTER_CONST_LONG("METRIC_PEAKABSOLUTEERROR", PeakAbsoluteErrorMetric);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("METRIC_PEAKSIGNALTONOISERATIO", PeakSignalToNoiseRatioMetric);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("METRIC_ROOTMEANSQUAREDERROR", RootMeanSquaredErrorMetric);
+#if MagickLibVersion >= 0x687
+	IMAGICK_REGISTER_CONST_LONG("METRIC_NORMALIZEDCROSSCORRELATIONERRORMETRIC", NormalizedCrossCorrelationErrorMetric);
+	IMAGICK_REGISTER_CONST_LONG("METRIC_FUZZERROR", FuzzErrorMetric);
+#endif
+#if MagickLibVersion >= 0x690
+	IMAGICK_REGISTER_CONST_LONG("METRIC_PERCEPTUALHASH_ERROR", PerceptualHashErrorMetric);
+#endif
+
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_CHAR", CharPixel);
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_DOUBLE", DoublePixel);
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_FLOAT", FloatPixel);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_INTEGER", IntegerPixel);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_LONG", LongPixel);
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_QUANTUM", QuantumPixel);
 	IMAGICK_REGISTER_CONST_LONG("PIXEL_SHORT", ShortPixel);
+	//For now we deliberately DO NOT support the longlong pixel type.
+	//64 bit integers and PHP do not play nicely.
+	//IMAGICK_REGISTER_CONST_LONG("PIXEL_LONGLONG", ..);
 	IMAGICK_REGISTER_CONST_LONG("EVALUATE_UNDEFINED", UndefinedEvaluateOperator);
 	IMAGICK_REGISTER_CONST_LONG("EVALUATE_ADD", AddEvaluateOperator);
 	IMAGICK_REGISTER_CONST_LONG("EVALUATE_AND", AndEvaluateOperator);
@@ -1118,13 +1357,19 @@ void php_imagick_initialize_constants(TSRMLS_D)
 #if MagickLibVersion >= 0x676
 	IMAGICK_REGISTER_CONST_LONG("EVALUATE_SUM", SumEvaluateOperator);
 #endif
+#if MagickLibVersion >= 0x690
+	IMAGICK_REGISTER_CONST_LONG("EVALUATE_ROOT_MEAN_SQUARE", RootMeanSquareEvaluateOperator);
+#endif
+
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_UNDEFINED", UndefinedColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_RGB", RGBColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_GRAY", GRAYColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_TRANSPARENT", TransparentColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_OHTA", OHTAColorspace);
+#if MagickLibVersion < 0x700
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_LAB", LABColorspace);
+#endif
 #endif
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_XYZ", XYZColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_YCBCR", YCbCrColorspace);
@@ -1137,13 +1382,15 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_HSB", HSBColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_HSL", HSLColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_HWB", HWBColorspace);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_REC601LUMA", Rec601LumaColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_REC709LUMA", Rec709LumaColorspace);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_LOG", LogColorspace);
 #if MagickLibVersion >= 0x642
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_CMY", CMYColorspace);
 #endif
-#if MagickLibVersion >= 0x679
+#if MagickLibVersion >= 0X679
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_LUV", LuvColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_HCL", HCLColorspace);
 #endif
@@ -1160,9 +1407,19 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_HCLP", HCLpColorspace);
 	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_YDBDR", YDbDrColorspace);
 #endif
+
+	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_REC601YCBCR", Rec601YCbCrColorspace);
+	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_REC709YCBCR", Rec709YCbCrColorspace);
+#if MagickLibVersion >= 0x690
+	IMAGICK_REGISTER_CONST_LONG("COLORSPACE_XYY", xyYColorspace);
+#endif
+
+
 	IMAGICK_REGISTER_CONST_LONG("VIRTUALPIXELMETHOD_UNDEFINED", UndefinedVirtualPixelMethod);
 	IMAGICK_REGISTER_CONST_LONG("VIRTUALPIXELMETHOD_BACKGROUND", BackgroundVirtualPixelMethod);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("VIRTUALPIXELMETHOD_CONSTANT", ConstantVirtualPixelMethod);  /* deprecated */
+#endif
 	IMAGICK_REGISTER_CONST_LONG("VIRTUALPIXELMETHOD_EDGE", EdgeVirtualPixelMethod);
 	IMAGICK_REGISTER_CONST_LONG("VIRTUALPIXELMETHOD_MIRROR", MirrorVirtualPixelMethod);
 	IMAGICK_REGISTER_CONST_LONG("VIRTUALPIXELMETHOD_TILE", TileVirtualPixelMethod);
@@ -1255,6 +1512,11 @@ void php_imagick_initialize_constants(TSRMLS_D)
 #if MagickLibVersion > 0x678
 	IMAGICK_REGISTER_CONST_LONG("RESOURCETYPE_THREAD", ThreadResource);
 #endif
+#if MagickLibVersion > 0x691
+	IMAGICK_REGISTER_CONST_LONG("RESOURCETYPE_WIDTH", WidthResource);
+	IMAGICK_REGISTER_CONST_LONG("RESOURCETYPE_HEIGHT", HeightResource);
+#endif
+
 	IMAGICK_REGISTER_CONST_LONG("DISPOSE_UNRECOGNIZED", UnrecognizedDispose);
 	IMAGICK_REGISTER_CONST_LONG("DISPOSE_UNDEFINED", UndefinedDispose);
 	IMAGICK_REGISTER_CONST_LONG("DISPOSE_NONE", NoneDispose);
@@ -1262,17 +1524,34 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("DISPOSE_PREVIOUS", PreviousDispose);
 #if MagickLibVersion > 0x631
 	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_UNDEFINED", UndefinedInterpolatePixel);
-	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_AVERAGE", AverageInterpolatePixel);
-	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_BICUBIC", BicubicInterpolatePixel);
-	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_BILINEAR", BilinearInterpolatePixel);
-	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_FILTER", FilterInterpolatePixel);
-	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_INTEGER", IntegerInterpolatePixel);
-	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_MESH", MeshInterpolatePixel);
+
+	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_AVERAGE", AverageInterpolatePixel); /* Average 4 nearest neighbours */
+#if MagickLibVersion < 0x700
+	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_BICUBIC", BicubicInterpolatePixel); /* Catmull-Rom interpolation */
+#endif
+	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_BILINEAR", BilinearInterpolatePixel); /* Triangular filter interpolation */
+#if MagickLibVersion < 0x700
+	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_FILTER", FilterInterpolatePixel); /* Use resize filter - (very slow) */
+#endif
+	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_INTEGER", IntegerInterpolatePixel); /* Integer (floor) interpolation */
+	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_MESH", MeshInterpolatePixel); /* Triangular mesh interpolation */
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_NEARESTNEIGHBOR", NearestNeighborInterpolatePixel);
 #endif
-#if MagickLibVersion > 0x634
-	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_SPLINE", SplineInterpolatePixel);
 #endif
+#if MagickLibVersion > 0x634
+	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_SPLINE", SplineInterpolatePixel); /* Cubic Spline (blurred) interpolation */
+#endif
+#if MagickLibVersion > 0x690
+	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_AVERAGE_9", Average9InterpolatePixel); /* Average 9 nearest neighbours */
+	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_AVERAGE_16", Average16InterpolatePixel); /* Average 16 nearest neighbours */
+	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_BLEND", BlendInterpolatePixel); /* blend of nearest 1, 2 or 4 pixels */
+	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_BACKGROUND_COLOR", BackgroundInterpolatePixel); /* just return background color */
+#if MagickLibVersion >= 0x687
+	IMAGICK_REGISTER_CONST_LONG("INTERPOLATE_CATROM", CatromInterpolatePixel); /* Catmull-Rom interpolation */
+#endif
+#endif
+
 #if MagickLibVersion > 0x628
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_UNDEFINED", UndefinedLayer);
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_COALESCE", CoalesceLayer);
@@ -1290,12 +1569,6 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_REMOVEDUPS", RemoveDupsLayer);
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_REMOVEZERO", RemoveZeroLayer);
 #endif
-#if MagickLibVersion >= 0x637
-	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_COMPOSITE", CompositeLayer);
-	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_MERGE", MergeLayer);
-	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_FLATTEN", FlattenLayer);
-	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_MOSAIC", MosaicLayer);
-#endif
 #if MagickLibVersion >= 0x644
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_TRIMBOUNDS", TrimBoundsLayer);
 #endif
@@ -1325,8 +1598,11 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("DISTORTION_POLAR", PolarDistortion);
 	IMAGICK_REGISTER_CONST_LONG("DISTORTION_DEPOLAR", DePolarDistortion);
 	IMAGICK_REGISTER_CONST_LONG("DISTORTION_BARREL", BarrelDistortion);
-	IMAGICK_REGISTER_CONST_LONG("DISTORTION_BARRELINVERSE", BarrelInverseDistortion);
 	IMAGICK_REGISTER_CONST_LONG("DISTORTION_SHEPARDS", ShepardsDistortion);
+
+	// SentinelDistortion is not a real distortion type. It is a guard value 
+	// that shouldn't have been exposed in Imagick. TODO remove at next
+	// minor version.
 	IMAGICK_REGISTER_CONST_LONG("DISTORTION_SENTINEL", SentinelDistortion);
 #endif
 #if MagickLibVersion >= 0x644
@@ -1350,8 +1626,9 @@ void php_imagick_initialize_constants(TSRMLS_D)
 #endif
 #if MagickLibVersion > 0x637
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_ACTIVATE", ActivateAlphaChannel);
-	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_DEACTIVATE", DeactivateAlphaChannel);
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_RESET", ResetAlphaChannel);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_SET", SetAlphaChannel);
 #endif
 #if MagickLibVersion > 0x645
@@ -1362,12 +1639,23 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_OPAQUE", OpaqueAlphaChannel);
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_SHAPE", ShapeAlphaChannel);
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_TRANSPARENT", TransparentAlphaChannel);
+#if MagickLibVersion >= 0x690
+	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_ASSOCIATE", AssociateAlphaChannel);
+	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_DISSOCIATE", DisassociateAlphaChannel);
+#endif
+
 	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_UNDEFINED", UndefinedColorInterpolate);
 	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_BARYCENTRIC", BarycentricColorInterpolate);
 	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_BILINEAR", BilinearColorInterpolate);
 	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_POLYNOMIAL", PolynomialColorInterpolate);
 	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_SPEPARDS", ShepardsColorInterpolate);
 	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_VORONOI", VoronoiColorInterpolate);
+#if MagickLibVersion >= 0x687
+	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_INVERSE", InverseColorInterpolate);
+#endif
+#if MagickLibVersion >= 0x693
+	IMAGICK_REGISTER_CONST_LONG("SPARSECOLORMETHOD_MANHATTAN", ManhattanColorInterpolate);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("DITHERMETHOD_UNDEFINED", UndefinedDitherMethod);
 	IMAGICK_REGISTER_CONST_LONG("DITHERMETHOD_NO", NoDitherMethod);
 	IMAGICK_REGISTER_CONST_LONG("DITHERMETHOD_RIEMERSMA", RiemersmaDitherMethod);
@@ -1384,7 +1672,9 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("FUNCTION_ARCTAN", ArctanFunction);
 #endif
 #if MagickLibVersion >= 0x680
+#if MagickLibVersion < 0x700
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_FLATTEN", FlattenAlphaChannel);
+#endif
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_REMOVE", RemoveAlphaChannel);
 #endif
 
@@ -1397,6 +1687,9 @@ void php_imagick_initialize_constants(TSRMLS_D)
 	IMAGICK_REGISTER_CONST_LONG("STATISTIC_MODE", ModeStatistic);
 	IMAGICK_REGISTER_CONST_LONG("STATISTIC_NONPEAK", NonpeakStatistic);
 	IMAGICK_REGISTER_CONST_LONG("STATISTIC_STANDARD_DEVIATION", StandardDeviationStatistic);
+#if MagickLibVersion >= 0x690
+	IMAGICK_REGISTER_CONST_LONG("STATISTIC_ROOT_MEAN_SQUARE", RootMeanSquareStatistic);
+#endif
 #endif
 
 #ifdef IMAGICK_WITH_KERNEL
@@ -1475,6 +1768,9 @@ IMAGICK_REGISTER_CONST_LONG("KERNEL_EUCLIDEAN", EuclideanKernel);
 IMAGICK_REGISTER_CONST_LONG("KERNEL_USER_DEFINED", UserDefinedKernel);
 IMAGICK_REGISTER_CONST_LONG("KERNEL_BINOMIAL", BinomialKernel);
 
+/* Draw directions */
+IMAGICK_REGISTER_CONST_LONG("DIRECTION_LEFT_TO_RIGHT", LeftToRightDirection);
+IMAGICK_REGISTER_CONST_LONG("DIRECTION_RIGHT_TO_LEFT", RightToLeftDirection);
 
 // The kernel is scaled directly using given scaling factor without change.
 IMAGICK_REGISTER_CONST_LONG("NORMALIZE_KERNEL_NONE", 0);
