@@ -29,6 +29,10 @@
 #  include <wand/MagickWand.h>
 #endif
 
+#if MagickLibVersion >= 0x700
+#include "shim_im6_to_im7.h"
+#endif
+
 /* Some extra headers */
 #include "Zend/zend_exceptions.h"
 #include "Zend/zend_interfaces.h"
@@ -36,6 +40,7 @@
 #include "ext/standard/info.h"
 #include "ext/standard/php_filestat.h"
 #include "php_ini.h"
+#include "Zend/zend.h"
 
 /* Include locale header */
 #ifdef HAVE_LOCALE_H
@@ -46,9 +51,30 @@
 	#define IMAGICK_WITH_KERNEL
 #endif
 
+
+
+#if MagickLibVersion >= 0x700
+#if !defined(MaxTextExtent)
+# define MaxTextExtent  4096  /* always >= 4096 */
+#endif
+
+#endif
+
+
+#ifdef ZEND_ENGINE_3
+	#define im_long zend_long
+#else
+	#define im_long long
+#endif
+
+
 typedef struct _php_imagick_callback {
 	void ***thread_ctx;
+#ifdef ZEND_ENGINE_3
+	zval user_callback;
+#else
 	zval *user_callback;
+#endif
 	struct _php_imagick_callback *previous_callback;
 } php_imagick_callback;
 
@@ -83,6 +109,7 @@ ZEND_EXTERN_MODULE_GLOBALS(imagick)
 #define zend_parse_parameters_none() zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "")
 #endif
 
+
 /* Class names */
 #define PHP_IMAGICK_SC_NAME "Imagick"
 #define PHP_IMAGICK_EXCEPTION_SC_NAME "ImagickException"
@@ -102,6 +129,18 @@ ZEND_EXTERN_MODULE_GLOBALS(imagick)
 #endif
 
 
+#ifdef ZEND_ENGINE_3
+
+/* Structure for Imagick object. */
+typedef struct _php_imagick_object  {
+	MagickWand *magick_wand;
+	char *progress_monitor_name;
+	zend_bool next_out_of_bound;
+	zend_object zo;
+} php_imagick_object;
+
+#else
+
 /* Structure for Imagick object. */
 typedef struct _php_imagick_object  {
 	zend_object zo;
@@ -110,6 +149,20 @@ typedef struct _php_imagick_object  {
 	zend_bool next_out_of_bound;
 } php_imagick_object;
 
+#endif
+
+
+#ifdef ZEND_ENGINE_3
+
+/* Structure for ImagickDraw object. */
+typedef struct _php_imagickdraw_object  {
+	DrawingWand *drawing_wand;
+	zend_object zo;
+} php_imagickdraw_object;
+
+
+#else 
+
 
 /* Structure for ImagickDraw object. */
 typedef struct _php_imagickdraw_object  {
@@ -117,6 +170,27 @@ typedef struct _php_imagickdraw_object  {
 	DrawingWand *drawing_wand;
 } php_imagickdraw_object;
 
+
+#endif
+
+#ifdef ZEND_ENGINE_3
+
+/* Structure for ImagickPixelIterator object. */
+typedef struct _php_imagickpixeliterator_object  {
+	
+	PixelIterator *pixel_iterator;
+	zend_bool initialized;
+
+#if MagickLibVersion <= 0x628
+	long rows;
+	long iterator_position;
+#endif
+
+	zend_object zo;
+} php_imagickpixeliterator_object;
+
+#else
+
 /* Structure for ImagickPixelIterator object. */
 typedef struct _php_imagickpixeliterator_object  {
 	zend_object zo;
@@ -127,8 +201,25 @@ typedef struct _php_imagickpixeliterator_object  {
 	long rows;
 	long iterator_position;
 #endif
+
 } php_imagickpixeliterator_object;
 
+#endif
+
+
+#ifdef ZEND_ENGINE_3
+
+/* Structure for ImagickPixel object. */
+typedef struct _php_imagickpixel_object  {
+	PixelWand *pixel_wand;
+	zend_bool initialized_via_iterator;
+	zend_object zo;
+} php_imagickpixel_object;
+
+
+#else
+
+
 /* Structure for ImagickPixel object. */
 typedef struct _php_imagickpixel_object  {
 	zend_object zo;
@@ -136,13 +227,109 @@ typedef struct _php_imagickpixel_object  {
 	zend_bool initialized_via_iterator;
 } php_imagickpixel_object;
 
+#endif
+
+
 #ifdef IMAGICK_WITH_KERNEL
+#ifdef ZEND_ENGINE_3
+/* Structure for ImagickKernel object. */
+typedef struct _php_imagickkernel_object  {
+	KernelInfo *kernel_info;
+	zend_object zo;
+} php_imagickkernel_object;
+#else
+
 /* Structure for ImagickKernel object. */
 typedef struct _php_imagickkernel_object  {
     zend_object zo;
     KernelInfo *kernel_info;
 } php_imagickkernel_object;
 #endif
+#endif
+
+//Object fetching.
+#ifdef ZEND_ENGINE_3 
+
+static inline php_imagick_object *php_imagick_fetch_object(zend_object *obj) {
+	return (php_imagick_object *)((char*)(obj) - XtOffsetOf(php_imagick_object, zo));
+}
+
+static inline php_imagickdraw_object *php_imagickdraw_fetch_object(zend_object *obj) {
+	return (php_imagickdraw_object *)((char*)(obj) - XtOffsetOf(php_imagickdraw_object, zo));
+}
+
+static inline php_imagickpixel_object *php_imagickpixel_fetch_object(zend_object *obj) {
+	return (php_imagickpixel_object *)((char*)(obj) - XtOffsetOf(php_imagickpixel_object, zo));
+}
+
+static inline php_imagickpixeliterator_object *php_imagickpixeliterator_fetch_object(zend_object *obj) {
+	return (php_imagickpixeliterator_object *)((char*)(obj) - XtOffsetOf(php_imagickpixeliterator_object, zo));
+}
+
+#ifdef IMAGICK_WITH_KERNEL
+static inline php_imagickkernel_object *php_imagickkernel_fetch_object(zend_object *obj) {
+	return (php_imagickkernel_object *)((char*)(obj) - XtOffsetOf(php_imagickkernel_object, zo));
+}
+#endif
+
+#else 
+	#define php_imagick_fetch_object(object) ((php_imagick_object *)object)
+	#define php_imagickdraw_fetch_object(object) ((php_imagickdraw_object *)object) 
+	#define php_imagickpixel_fetch_object(object) ((php_imagickpixel_object *)object)
+	#define php_imagickpixeliterator_fetch_object(object) ((php_imagickpixeliterator_object *)object)
+	#define php_imagickkernel_fetch_object(object) ((php_imagickkernel_object *)object)
+#endif
+
+// Object access
+#ifdef ZEND_ENGINE_3
+	#define Z_IMAGICK_P(zv) php_imagick_fetch_object(Z_OBJ_P((zv)))
+	#define Z_IMAGICKDRAW_P(zv) php_imagickdraw_fetch_object(Z_OBJ_P((zv)))
+	#define Z_IMAGICKPIXEL_P(zv) php_imagickpixel_fetch_object(Z_OBJ_P((zv)))
+	#define Z_IMAGICKPIXELITERATOR_P(zv) php_imagickpixeliterator_fetch_object(Z_OBJ_P((zv)))
+#ifdef IMAGICK_WITH_KERNEL
+	#define Z_IMAGICKKERNEL_P(zv) php_imagickkernel_fetch_object(Z_OBJ_P((zv)))
+#endif
+#else
+	#define Z_IMAGICK_P(zv) (php_imagick_object *)zend_object_store_get_object(zv TSRMLS_CC)
+	#define Z_IMAGICKDRAW_P(zv) (php_imagickdraw_object *)zend_object_store_get_object(zv TSRMLS_CC)
+	#define Z_IMAGICKPIXEL_P(zv) (php_imagickpixel_object *) zend_object_store_get_object(zv TSRMLS_CC)
+	#define Z_IMAGICKPIXELITERATOR_P(zv) (php_imagickpixeliterator_object *)zend_object_store_get_object(zv TSRMLS_CC)
+	#define Z_IMAGICKKERNEL_P(zv) (php_imagickkernel_object *)zend_object_store_get_object(zv TSRMLS_CC)
+#endif
+
+// String access
+#ifdef ZEND_ENGINE_3
+	#define IM_ZVAL_STRING(zv, charstar) ZVAL_STRING(zv, charstar);
+	#define IM_RETURN_STRING(s) RETURN_STRING(s)
+#else
+	#define IM_ZVAL_STRING(zv, charstar) ZVAL_STRING(zv, charstar, 1);
+	//RETURN_STRING(s, duplicate) 
+	#define IM_RETURN_STRING(s) RETURN_STRING(s, 0)
+#endif
+
+#ifdef ZEND_ENGINE_3
+	#define IM_add_assoc_string(zv, key, charstr) add_assoc_string(zv, key, charstr)
+	#define IM_ZVAL_STRINGL(zv, charstr, length) ZVAL_STRINGL(zv, charstr, length)
+	#define IM_add_next_index_string(zv, charstr) add_next_index_string(zv, charstr)
+	#define IM_LEN_TYPE size_t
+#else
+	#define IM_add_assoc_string(zv, key, charstr) add_assoc_string(zv, key, charstr, 1)
+	#define IM_add_assoc_string(zv, key, charstr) add_assoc_string(zv, key, charstr, 1)
+	#define IM_ZVAL_STRINGL(zv, charstr, length) ZVAL_STRINGL(zv, charstr, length, 1)
+	#define IM_add_next_index_string(zv, charstr) add_next_index_string(zv, charstr, 1)
+	#define IM_add_next_index_zval(zv1, zv2)
+	#define IM_LEN_TYPE int
+#endif
+
+#ifdef ZEND_ENGINE_3
+	#define IM_ZEND_OBJECT zend_object
+#else 
+	#define IM_ZEND_OBJECT void
+#endif
+
+
+
+
 
 /* Define some color constants */
 typedef enum _php_imagick_color_t {
@@ -154,7 +341,9 @@ typedef enum _php_imagick_color_t {
 	PHP_IMAGICK_COLOR_RED,
 	PHP_IMAGICK_COLOR_YELLOW,
 	PHP_IMAGICK_COLOR_MAGENTA,
+#if MagickLibVersion < 0x700
 	PHP_IMAGICK_COLOR_OPACITY,
+#endif
 	PHP_IMAGICK_COLOR_ALPHA,
 	PHP_IMAGICK_COLOR_FUZZ,
 	PHP_IMAGICK_COLOR_MAX,
@@ -224,13 +413,17 @@ PHP_METHOD(imagick, adaptiveblurimage);
 PHP_METHOD(imagick, contraststretchimage);
 PHP_METHOD(imagick, adaptivesharpenimage);
 PHP_METHOD(imagick, randomthresholdimage);
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, roundcornersimage);
+#endif
 PHP_METHOD(imagick, setiteratorindex);
 PHP_METHOD(imagick, getiteratorindex);
 PHP_METHOD(imagick, transformimage);
 #endif
 #if MagickLibVersion > 0x630
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, setimageopacity);
+#endif
 PHP_METHOD(imagick, orderedposterizeimage);
 #endif
 #if MagickLibVersion > 0x631
@@ -262,13 +455,18 @@ PHP_METHOD(imagick, distortimage);
 PHP_METHOD(imagick, writeimagefile);
 PHP_METHOD(imagick, writeimagesfile);
 PHP_METHOD(imagick, resetimagepage);
+
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, setimageclipmask);
 PHP_METHOD(imagick, getimageclipmask);
+#endif
 PHP_METHOD(imagick, animateimages);
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, recolorimage);
 #endif
 #endif
+#endif
 #if MagickLibVersion > 0x636
 PHP_METHOD(imagick, setfont);
 PHP_METHOD(imagick, getfont);
@@ -406,7 +604,9 @@ PHP_METHOD(imagick, charcoalimage);
 PHP_METHOD(imagick, normalizeimage);
 PHP_METHOD(imagick, oilpaintimage);
 PHP_METHOD(imagick, posterizeimage);
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, radialblurimage);
+#endif
 PHP_METHOD(imagick, raiseimage);
 PHP_METHOD(imagick, resampleimage);
 PHP_METHOD(imagick, resizeimage);
@@ -419,6 +619,9 @@ PHP_METHOD(imagick, shadowimage);
 PHP_METHOD(imagick, setimageattribute);
 #endif
 PHP_METHOD(imagick, setimagebackgroundcolor);
+#if MagickLibVersion >= 0x700
+PHP_METHOD(imagick, setimagechannelmask);
+#endif
 PHP_METHOD(imagick, setimagecompose);
 PHP_METHOD(imagick, setimagecompression);
 PHP_METHOD(imagick, setimagedelay);
@@ -428,7 +631,9 @@ PHP_METHOD(imagick, setimageiterations);
 PHP_METHOD(imagick, setimagemattecolor);
 PHP_METHOD(imagick, setimagepage);
 PHP_METHOD(imagick, setimageprogressmonitor);
+#if MagickLibVersion > 0x653
 PHP_METHOD(imagick, setprogressmonitor);
+#endif
 PHP_METHOD(imagick, setimageresolution);
 PHP_METHOD(imagick, setimagescene);
 PHP_METHOD(imagick, setimagetickspersecond);
@@ -446,17 +651,22 @@ PHP_METHOD(imagick, queryfonts);
 PHP_METHOD(imagick, queryfontmetrics);
 PHP_METHOD(imagick, steganoimage);
 PHP_METHOD(imagick, motionblurimage);
+#if MagickLibVersion < 0x700
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, mosaicimages);
 #endif
+#endif
 PHP_METHOD(imagick, morphimages);
 PHP_METHOD(imagick, minifyimage);
 PHP_METHOD(imagick, addnoiseimage);
 PHP_METHOD(imagick, affinetransformimage);
+#if MagickLibVersion < 0x700
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, averageimages);
 #endif
+#endif
 PHP_METHOD(imagick, borderimage);
+PHP_METHOD(imagick, calculatecrop);
 PHP_METHOD(imagick, chopimage);
 PHP_METHOD(imagick, clipimage);
 PHP_METHOD(imagick, clippathimage);
@@ -478,9 +688,14 @@ PHP_METHOD(imagick, embossimage);
 PHP_METHOD(imagick, enhanceimage);
 PHP_METHOD(imagick, equalizeimage);
 PHP_METHOD(imagick, evaluateimage);
+#if MagickLibVersion >= 0x687
+PHP_METHOD(imagick, evaluateimages);
+#endif
+#if MagickLibVersion < 0x700
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, flattenimages);
 #endif
+#endif
 PHP_METHOD(imagick, flipimage);
 PHP_METHOD(imagick, flopimage);
 #if MagickLibVersion >= 0x655
@@ -490,9 +705,11 @@ PHP_METHOD(imagick, frameimage);
 PHP_METHOD(imagick, fximage);
 PHP_METHOD(imagick, gammaimage);
 PHP_METHOD(imagick, gaussianblurimage);
+#if MagickLibVersion < 0x700
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, getimageattribute);
 #endif
+#endif
 PHP_METHOD(imagick, getimagebackgroundcolor);
 PHP_METHOD(imagick, getimageblueprimary);
 PHP_METHOD(imagick, getimagebordercolor);
@@ -547,7 +764,9 @@ PHP_METHOD(imagick, magnifyimage);
 PHP_METHOD(imagick, mapimage);
 PHP_METHOD(imagick, mattefloodfillimage);
 #endif
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, medianfilterimage);
+#endif
 PHP_METHOD(imagick, negateimage);
 #if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, paintopaqueimage);
@@ -557,12 +776,18 @@ PHP_METHOD(imagick, previewimages);
 PHP_METHOD(imagick, profileimage);
 PHP_METHOD(imagick, quantizeimage);
 PHP_METHOD(imagick, quantizeimages);
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, reducenoiseimage);
+#endif
 PHP_METHOD(imagick, removeimageprofile);
 PHP_METHOD(imagick, separateimagechannel);
 PHP_METHOD(imagick, sepiatoneimage);
+
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, setimagebias);
 PHP_METHOD(imagick, setimagebiasquantum);
+#endif
+
 PHP_METHOD(imagick, setimageblueprimary);
 PHP_METHOD(imagick, setimagebordercolor);
 PHP_METHOD(imagick, setimagechanneldepth);
@@ -590,6 +815,10 @@ PHP_METHOD(imagick, newpseudoimage);
 PHP_METHOD(imagick, getcompression);
 PHP_METHOD(imagick, getcompressionquality);
 PHP_METHOD(imagick, getcopyright);
+PHP_METHOD(imagick, getconfigureoptions);
+#if MagickLibVersion > 0x660
+PHP_METHOD(imagick, getfeatures);
+#endif
 PHP_METHOD(imagick, getfilename);
 PHP_METHOD(imagick, getformat);
 PHP_METHOD(imagick, gethomeurl);
@@ -597,6 +826,7 @@ PHP_METHOD(imagick, getinterlacescheme);
 PHP_METHOD(imagick, getoption);
 PHP_METHOD(imagick, getpackagename);
 PHP_METHOD(imagick, getpage);
+PHP_METHOD(imagick, gethdrienabled);
 PHP_METHOD(imagick, getquantum);
 PHP_METHOD(imagick, getquantumdepth);
 PHP_METHOD(imagick, getquantumrange);
@@ -641,7 +871,20 @@ PHP_METHOD(imagick, morphology);
 PHP_METHOD(imagick, filter);
 PHP_METHOD(imagick, setantialias);
 PHP_METHOD(imagick, getantialias);
+#if MagickLibVersion > 0x676
+PHP_METHOD(imagick, colordecisionlistimage);
+#endif
 
+#if MagickLibVersion >= 0x660
+PHP_METHOD(imagick, autogammaimage);
+#endif //#if MagickLibVersion >= 0x660
+#if MagickLibVersion >= 0x692
+PHP_METHOD(imagick, autoorient);
+PHP_METHOD(imagick, compositeimagegravity);
+#endif // #if MagickLibVersion >= 0x692
+#if MagickLibVersion >= 0x692
+PHP_METHOD(imagick, localcontrastimage);
+#endif // #if MagickLibVersion >= 0x692
 
 /* Forward declarations (ImagickDraw) */
 #if MagickLibVersion > 0x628
@@ -689,7 +932,13 @@ PHP_METHOD(imagickdraw, translate);
 PHP_METHOD(imagickdraw, line);
 PHP_METHOD(imagickdraw, polygon);
 PHP_METHOD(imagickdraw, arc);
+
+#if MagickLibVersion >= 0x700
+PHP_METHOD(imagickdraw, alpha);
+#else
 PHP_METHOD(imagickdraw, matte);
+#endif
+
 PHP_METHOD(imagickdraw, point);
 PHP_METHOD(imagickdraw, getfontstretch);
 PHP_METHOD(imagickdraw, setfontstretch);
@@ -774,6 +1023,25 @@ PHP_METHOD(imagickdraw, setstrokeopacity);
 PHP_METHOD(imagickdraw, setvectorgraphics);
 PHP_METHOD(imagickdraw, pop);
 PHP_METHOD(imagickdraw, push);
+PHP_METHOD(imagickdraw, getopacity);
+PHP_METHOD(imagickdraw, setopacity);
+
+#if MagickLibVersion >= 0x675
+PHP_METHOD(imagickdraw, getfontresolution);
+PHP_METHOD(imagickdraw, setfontresolution);
+PHP_METHOD(imagickdraw, getbordercolor);
+PHP_METHOD(imagickdraw, setbordercolor);
+#endif //#if MagickLibVersion >= 0x675
+
+#if MagickLibVersion >= 0x693
+PHP_METHOD(imagickdraw, setdensity);
+PHP_METHOD(imagickdraw, getdensity);
+#endif// #if MagickLibVersion >= 0x693
+
+#if MagickLibVersion >= 0x692
+PHP_METHOD(imagickdraw, gettextdirection);
+PHP_METHOD(imagickdraw, settextdirection);
+#endif //#if MagickLibVersion >= 0x692
 
 /* Imagick Pixel iterator */
 PHP_METHOD(imagickpixeliterator, __construct);
@@ -817,6 +1085,9 @@ PHP_METHOD(imagickpixel, getcolorasstring);
 PHP_METHOD(imagickpixel, getcolorcount);
 PHP_METHOD(imagickpixel, setcolorcount);
 PHP_METHOD(imagickpixel, clone);
+#if MagickLibVersion >= 0x693
+PHP_METHOD(imagickpixel, setcolorfrompixel);
+#endif //#if MagickLibVersion >= 0x693
 
 #ifdef IMAGICK_WITH_KERNEL
 PHP_METHOD(imagickkernel, frommatrix);
@@ -828,5 +1099,16 @@ PHP_METHOD(imagickkernel, scale);
 PHP_METHOD(imagickkernel, addunitykernel);
 #endif
 
+#if MagickLibVersion < 0x700
+#define KernelValueType double
+#else
+#define KernelValueType MagickRealType
+#endif
+
+#if MagickLibVersion >= 0x700
+#define IM_DEFAULT_CHANNEL UndefinedChannel
+#else
+#define IM_DEFAULT_CHANNEL DefaultChannels
+#endif
 
 #endif /* PHP_IMAGICK_DEFS_H */
